// k_string.hh
// Library of string manipulation functions.
// Kevin Puckett Albrecht
// Rev 58
// 3 June 2000 - 25 Oct 2000

#ifndef K_STRING_HH
#define K_STRING_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

#define K_NUMERICS "0123456789-."
#define K_DIGITS "0123456789"
#define K_LETTERS "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
#define K_SPECIAL1 "~`!@#$%^&*()_-+={[}]|\\:;\"'<,>.?/"
#define K_SPECIAL2 "*()-+=:;\"<,>./&"
#define K_LOWERCASE "abcdefghijklmnopqrstuvwxyz"
#define K_UPPERCASE "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
#define K_WHITESPACE "\t\0\n "

#define K_STRING_K_SWAP_STRING_SIZE 256
#define K_STRING_K_FLOAT_STR_SIZE 256
#define K_STRING_K_INT_STR_SIZE 256
#define K_STRING_K_POS_TEMP_SIZE 256

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <string.h>
#include <stdlib.h>

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

char *k_app (char *one, char *two);
void k_cat (char *one, char *two);
void k_catd (char *one, char *two);
char k_char (char *str, int pos);
char *k_clone (char *as);
void k_copy (char *one, char *two);
void k_copyd (char *one, char *two);
void k_del (char *str);
char *k_float (float f, int places = 10);
bool k_inSet (char ch, char *str);
char *k_int (int n);
int k_pos (char *str, char ch, int after = 0);
int k_pos (char *str, char *sub, int after = 0);
bool k_same (char *one, char *two);
char *k_sub (char *str, int one, int two);
char *k_subd (char *str, int one, int two);
char *k_swap (char *str, char *one, char *two);
char *k_trim (char *str);
char *k_trimd (char *str);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

//// k_app ////
//// Outputs the concatenation of 'one' and 'two'           ////
//// If the inputs are NULL, they are given an empty string ////
////////////////////////////////////////////////////////////////
char *k_app (char *one, char *two)
{
  if (one == NULL) strcpy (one, "");
  if (two == NULL) strcpy (two, "");

  char *temp = new char[strlen(one) + strlen(two)];
  k_copy (temp, one);
  k_cat (temp, two);
  return temp;
}

//// k_cat ////
//// Concatenates 'two' to the end of 'one' ////
////////////////////////////////////////////////
void k_cat (char *one, char *two)
{
  strcat (one, two);
}

//// k_catd ////
//// Concatenates 'two' to the end of 'one', then deletes 'two' ////
////////////////////////////////////////////////////////////////////
void k_catd (char *one, char *two)
{
  strcat (one, two);
  delete [] two;
}

//// k_char ////
//// Returns the char in the specified position in the specified string ////
////////////////////////////////////////////////////////////////////////////
char k_char (char *str, int pos)
{
  if ((pos < 0) || (pos >= (signed) strlen (str)))
  {
    // out of bounds //
    return 0;
  }
  return str[pos];
}

//// k_clone ////
char *k_clone (char *as)
{
  char *temp = new char[strlen(as) + 1];
  k_copy (temp, as);
  return temp;
}

//// k_copy ////
//// Copies 'two' into 'one' ////
/////////////////////////////////
void k_copy (char *one, char *two)
{
  if (two == NULL) strcpy (one, "");
  else             strcpy (one, two);
}

//// k_copyd ////
//// Copies 'two' into 'one', then deletes 'two' ////
/////////////////////////////////////////////////////
void k_copyd (char *one, char *two)
{
  if (two == NULL) strcpy (one, "");
  else             strcpy (one, two);
  k_del (two);
}

//// k_del ////
//// Deletes the string if it is not null or of zero len ////
/////////////////////////////////////////////////////////////
void k_del (char *str)
{
  if (str == NULL) return; //Rev 52.6//

  if (strlen (str) > 0)
  {
    delete [] str;
  }
}

//// k_float ////
//// Returns the string equivalent of the float ////
////////////////////////////////////////////////////
char *k_float (float f, int places)
{
  if (f == 0) return "0";

  int dec, sign;

  #ifdef ENV_GCC
    char *s = fcvt ((double) f, places, &dec, &sign);
  #endif
  #ifndef ENV_GCC
    char *s = _fcvt ((double) f, places, &dec, &sign);
  #endif

  char *buf = new char[K_STRING_K_FLOAT_STR_SIZE];
  if (sign != 0) k_copy (buf, "-");
  else           k_copy (buf, "");

  if (dec <= 0)
  {
    k_cat (buf, ".");
    while (dec < 0)
    {
      k_cat (buf, "0");
      dec++;
    }
    k_cat (buf, s);
  }
  else
  {
    k_cat (buf, k_sub (s, 0, dec));
    k_cat (buf, ".");
    k_cat (buf, k_sub (s, dec, strlen (s)));
  }

  for (int i = strlen (buf) - 1; i >= 0; i--)
  {
    switch (buf[i])
    {
      case '0':
        buf[i] = '\0';
        break;
      case '.':
        buf[i] = '\0';
        goto OUT;
      default:
        goto OUT;
    }
  }
  OUT:

  return buf;
}

//// k_inSet ////
//// Returns 'true' if 'ch' is in 'str' ////
////////////////////////////////////////////
bool k_inSet (char ch, char *str)
{
  if (k_pos (str, ch) < 0)
    return false;
  else
    return true;
}

//// k_int ////
//// Returns the string equivalent of the integer ////
//////////////////////////////////////////////////////
char *k_int (int n)
{
  char *s = new char[K_STRING_K_INT_STR_SIZE];
  k_copy (s, "");
  itoa (n, s, 10);
  return s;
}

//// k_pos ////
//// Returns the position of the first occurance of the char after 'after'; ////
////  returns -1 if not found                                               ////
////////////////////////////////////////////////////////////////////////////////
int k_pos (char *str, char ch, int after)
{
  if ((after >= (signed) strlen (str)) || (after < 0))
  {
    // out of bounds //
    return -1;
  }

  for (int i = after; i < (signed) strlen (str); i++)
  {
    if (str[i] == ch)
    {
      // found //
      return i;
    }
  }
  
  // not found //
  return -1;
}

//// k_pos ////
//// Returns the position of the first occurance of the string after 'after'; ////
////  returns -1 if not found                                                 ////
//////////////////////////////////////////////////////////////////////////////////
int k_pos (char *str, char *sub, int after)
{
  if ((after >= (signed) strlen (str)) || (after < 0) || ((signed) strlen (sub) > (signed) strlen (str)))
  {
    // out of bounds //
    return -1;
  }

  // variables //
  int str_len = strlen (str);
  int sub_len = strlen (sub);
  int temp_len = 0;
  char *temp = new char[K_STRING_K_POS_TEMP_SIZE];

  // determine if substring is present:
  for (int str_pos = after; str_pos <= str_len; str_pos++)
  {
    if (str[str_pos] == sub[0])
    {
      for (int sub_pos = 0; sub_pos < sub_len; sub_pos++)
      {
        temp[sub_pos] = str[str_pos + sub_pos];
        temp_len = sub_pos;
      }
      temp[temp_len + 1] = '\0';
      if (k_same (temp, sub))
      {
        k_del (temp);
        return str_pos;
      }
    }
  }

  k_del (temp);
  return -1;
}

//// k_pos ////
//// Returns 'true' if 'one' and 'two' are the same ////
////////////////////////////////////////////////////////
bool k_same (char *one, char *two)
{
  return (strcmp (one, two) == 0);
}

//// k_sub ////
//// Returns the substring between two points; '0' if out of bounds ////
////////////////////////////////////////////////////////////////////////
char *k_sub (char *str, int one, int two)
{
  int str_len = strlen (str);
  char *sub = new char[two - one + 1];
  strcpy (sub, "");

  if ((one < 0) || (one > str_len) || (two < one))
  {
    // out of bounds //
    return 0;
  }
  if (one == two)
  {
    // zero length //
    return "";
  }

  // create substring //
  for (int str_pos = one, sub_pos = 0; str_pos < two; str_pos++, sub_pos++)
  {
    sub[sub_pos] = str[str_pos];
  }
  sub[two - one] = '\0'; // end with '\0'
  
  return sub;
}

//// k_sub ////
//// Returns the substring between two points; '0' if out of bounds; ////
////  deletes the argument                                           ////
/////////////////////////////////////////////////////////////////////////
char *k_subd (char *str, int one, int two)
{
  int str_len = strlen (str);
  char *sub = new char[two - one + 1];
  strcpy (sub, "");

  if ((one < 0) || (one > str_len) || (two < one))
  {
    // out of bounds //
    k_del (sub);
    k_del (str);
    return 0;
  }
  if (one == two)
  {
    // zero length //
    k_del (sub);
    k_del (str);
    return "";
  }

  // create substring //
  for (int str_pos = one, sub_pos = 0; str_pos < two; str_pos++, sub_pos++)
  {
    sub[sub_pos] = str[str_pos];
  }
  sub[two - one] = '\0'; // end with '\0'

  k_del (str);
  return sub;
}


//// k_swap ////
//// Swaps all the characters in a string that occur in the first array with matching ////
////  characters in the second array; returns the string with swapped characters;     ////
////  '0' if errors                                                                   ////
//////////////////////////////////////////////////////////////////////////////////////////
char *k_swap (char *str, char *one, char *two)
{
  int str_len = strlen (str);
  int array_len = strlen (one);
  char *swapped = new char[K_STRING_K_SWAP_STRING_SIZE/*str_len might be zero*/];
  
  strcpy (swapped, str);

  if (strlen (one) != strlen (two))
  {
    // arrays do not match //
    k_del (swapped);
    return 0;
  }

  for (int str_pos = 0; str_pos < str_len; str_pos++)
  {
    for (int array_pos = 0; array_pos < array_len ; array_pos++)
    {
      if (swapped[str_pos] == one[array_pos])
      {
        swapped[str_pos] = two[array_pos];
      }
    }
  }
  
  return swapped;
}

//// k_trim ////
//// Returns the string trimmed of white space ////
///////////////////////////////////////////////////
char *k_trim (char *str)
{
  int end = strlen (str);
  int start = 0;

  while ((start < end) && (str[start] <= ' '))
  {
    start++;
  }

  while ((start < end) && (str[end - 1] <= ' '))
  {
    end--;
  }

  if ((start > 0) || (end < (signed) strlen (str)))
  {
    return k_sub (str, start, end);
  }
  else
  {
    char *temp = new char[strlen (str)];
    strcpy (temp, str);
    return temp;
  }
}

//// k_trim ////
//// Returns the string trimmed of white space; deletes the arg ////
////////////////////////////////////////////////////////////////////
char *k_trimd (char *str)
{
  int end = strlen (str);
  int start = 0;

  while ((start < end) && (str[start] <= ' '))
  {
    start++;
  }

  while ((start < end) && (str[end - 1] <= ' '))
  {
    end--;
  }

  if ((start > 0) || (end < (signed) strlen (str)))
  {
    char *temp = k_subd (str, start, end);
    return temp;
  }
  else
  {
    return str;
  }
}

#endif