// scanner.hh
// Scanning Class
// Kevin Puckett Albrecht
// Rev 6
// 21 March 2000 - 28 April 2001

#ifndef SCANNER_HH
#define SCANNER_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "inbuffer.hh"
#include "common.hh"

#include "token.hh"
#include "eoftoken.hh"
#include "eoltoken.hh"
#include "errortoken.hh"
#include "numbertoken.hh"
#include "specialtoken.hh"
#include "stringtoken.hh"
#include "wordtoken.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

class Scanner
{
  public:
    Scanner () {}
    ~Scanner () {}
    Token *Get ();

  private:
    WordToken    wordToken;
    NumberToken  numberToken;
    StringToken  stringToken;
    SpecialToken specialToken;
    EOLToken     eolToken;
    EOFToken     eofToken;
    ErrorToken   errorToken;

    void SkipBlankSpaces ();
};

//// Get ////
//// Get next token from buffer
///////////////////////////////
Token *Scanner::Get ()
{
  Token *pToken; // ptr to token to return
 
  SkipBlankSpaces();

  char c = gpBuffer->Get ();
  if (k_inSet (c, K_LETTERS)) pToken = &wordToken;
  else if (k_inSet (c, K_DIGITS)) pToken = &numberToken;
  else if (c == '\"') pToken = &stringToken;
  else if (k_inSet (c, "?*()-+=:\"<,>./&@")) pToken = &specialToken;
  else if ((c == 0) || (c == ';')) pToken = &eolToken;
  else if (c == INBUFFER_EOF) pToken = &eofToken;
  else {pToken = &errorToken;}

  // Extract a token of that class, and return a pointer to it //
  pToken->Get ();
  return pToken;
}

//// SkipBlankSpaces ////
//// Remove blank spaces from buffer; includes comments, and all whitespace
///////////////////////////////////////////////////////////////////////////
void Scanner::SkipBlankSpaces ()
{
  while (((gpBuffer->Get() <= ' ') && (gpBuffer->Get() > 0)) || (gpBuffer->Get() == '['))
  {
    if (gpBuffer->Get() == '[')
    {
      gpBuffer->Seek ();
      if (gpBuffer->Get() == '[') // single line comment
      {
        while (gpBuffer->Get() != 0)
        {
          gpBuffer->Seek ();
        }
        if (gpBuffer->Get() == 0)
        {
          // DEBUGGED 6/29/00 //
          // By returning now, Scanner::Get() can return an EOLToken //
          return;
        }
      }
      else // multiple line comment
      {
        int nLine = gpCommon->nCurrentLine;
        while (gpBuffer->Get() != ']')
        {
          if (gpBuffer->Get() == INBUFFER_EOF)
          {
            gpCommon->nCurrentLine = nLine;
            gpErrorS->Show (EFF_TERMINATE, ES_INCOMPLETE_COMMENT);
          }
          gpBuffer->Seek ();
        }
      }
    }
    gpBuffer->Seek ();
  }
}

#endif
