// stringtoken.hh
// string token class
// Kevin Puckett Albrecht
// Rev 4
// 21 March 2000 - 27 June 2000

#ifndef STRINGTOKEN_HH
#define STRINGTOKEN_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <string.h>

// Local //
#include "k_string.hh"
#include "token.hh"
#include "error.hh"
#include "inbuffer.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

class StringToken : public Token
{
  public:
    StringToken () {code = codeString;}
    virtual void Get ();
    virtual bool IsDelimiter () const {return true;}
};

void StringToken::Get ()
{
  char cCurr; // current character
  int pos = 0; // ptr to char in string

  buffer[pos++] = '\"';  // opening quote

  // Get the string //
  gpBuffer->Seek (); // skip opening quote
  cCurr = gpBuffer->Get ();
  while (cCurr != INBUFFER_EOF)
  {
    if (cCurr == '\"') // current char is quote
    {
      gpBuffer->Seek ();
      cCurr = gpBuffer->Get ();
      if (cCurr != '\"') 
      {
        break; // not a double quote, so end the string
      }
    }
    else if (cCurr == 0) // end of line reached
    {
      cCurr = ' '; // replace end-line with blank
    }

    // Append current char to string //
    buffer[pos++] = cCurr;

    // Get next char //
    gpBuffer->Seek ();
    cCurr = gpBuffer->Get ();
  }

  /**** Should possibly be removed, since the first char after a string may be eof ****/
  if (cCurr == INBUFFER_EOF)
  {
    gpErrorS->Show (EFF_IGNORE, ES_UNEXPECTED_EOF);
  }
  /**************************************************************/

  buffer[pos++] = '\"'; // closing quote
  buffer[pos] = '\0';

  // After this, this->buffer will contain the string with quotes and //
  //  this->value will contain the string minus the quotes            //
  //NOTE: the constant specifier maybe should be true?//
  SetContent (tValue, false, k_sub (buffer, 1, strlen (buffer) - 1), 0);
}

#endif
