// wordtoken.hh
// word token class
// Kevin Puckett Albrecht
// Rev 14
// 20 March 2000 - 6 May 2001

#ifndef WORDTOKEN_HH
#define WORDTOKEN_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

#define WORDTOKEN_MIN_WORD_LEN 2
#define WORDTOKEN_MAX_WORD_LEN 8

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <string.h>

// Local //
#include "k_string.hh"
#include "token.hh"
#include "inbuffer.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

class WordToken : public Token
{
  public:
    virtual void Get ();
    virtual bool IsDelimiter () const {return false;}

  private:
    void SetCode ();
};

//// Get ////
void WordToken::Get ()
{
  char c = gpBuffer->Get ();
  int nBufferPos = 0;

  do {
    buffer[nBufferPos++] = c;
    gpBuffer->Seek();
    c = gpBuffer->Get ();
  } while (k_inSet (c, K_LETTERS) || k_inSet (c, K_DIGITS));

  buffer[nBufferPos] = '\0';

  SetCode ();
}

//// SetCode ////
void WordToken::SetCode ()
{
  int nLen = strlen (buffer);

  code = codeIdentifier; // first assume it's an identifier

  // is it the right length? //
  if ((nLen >= WORDTOKEN_MIN_WORD_LEN) && (nLen <= WORDTOKEN_MAX_WORD_LEN))
  {
    switch (nLen)
    {
      case 2:
        if (strcmp (buffer, "if") == 0) code = codeIF;
        else if (strcmp (buffer, "or") == 0) code = codeOR;
        else if (strcmp (buffer, "is") == 0) code = codeIS;
        break;
      case 3:
        if (strcmp (buffer, "int") == 0) code = codeINT;
        else if (strcmp (buffer, "rat") == 0) code = codeRAT;
        else if (strcmp (buffer, "str") == 0) code = codeSTR;
        else if (strcmp (buffer, "end") == 0) code = codeEND;
        else if (strcmp (buffer, "and") == 0) code = codeAND;
        else if (strcmp (buffer, "not") == 0) code = codeNOT;
        else if (strcmp (buffer, "var") == 0) code = codeVAR;
        else if (strcmp (buffer, "fin") == 0) code = codeFIN;
        else if (strcmp (buffer, "put") == 0) code = codePUT;
        break;
      case 4:
        if (strcmp (buffer, "bool") == 0) code = codeBOOL;
        else if (strcmp (buffer, "fout") == 0) code = codeFOUT;
        else if (strcmp (buffer, "else") == 0) code = codeELSE;
        else if (strcmp (buffer, "loop") == 0) code = codeLOOP;
        else if (strcmp (buffer, "goto") == 0) code = codeGOTO;
        else if (strcmp (buffer, "quit") == 0) code = codeQUIT;
        else if (strcmp (buffer, "sget") == 0) code = codeSGET;
        else if (strcmp (buffer, "fput") == 0) code = codeFPUT;
        else if (strcmp (buffer, "putn") == 0) code = codePUTN;
        break;
      case 5:
        if (strcmp (buffer, "block") == 0) code = codeBLOCK;
        else if (strcmp (buffer, "const") == 0) code = codeCONST;
        else if (strcmp (buffer, "isnot") == 0) code = codeISNOT;
        else if (strcmp (buffer, "snext") == 0) code = codeSNEXT;
        else if (strcmp (buffer, "szero") == 0) code = codeSZERO;
        else if (strcmp (buffer, "fputn") == 0) code = codeFPUTN;
        else if (strcmp (buffer, "mrand") == 0) code = codeMRAND;
        break;
      case 6:
        if (strcmp (buffer, "return") == 0) code = codeRETURN;
        else if (strcmp (buffer, "elseif") == 0) code = codeELSEIF;
        else if (strcmp (buffer, "fclose") == 0) code = codeFCLOSE;
        break;
      case 7:
        if (strcmp (buffer, "getline") == 0) code = codeGETLINE;
        break;
      case 8:
        if (strcmp (buffer, "implicit") == 0) code = codeIMPLICIT;
        else if (strcmp (buffer, "explicit") == 0) code = codeEXPLICIT;
        else if (strcmp (buffer, "fgetline") == 0) code = codeFGETLINE;
        break;
    }
  }
}

#endif
